@@ -285,7 +285,7 @@ impl DnsConn {
                 }
             }
 
-            log::trace!("Recv bytes {:?} from {}", &b[..n], src);
+            //log::trace!("Recv bytes {:?} from {}", &b[..n], src);
 
             let mut p = Parser::default();
             if let Err(err) = p.start(&b[..n]) {
